package com.fusi24.rfid;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;

import com.fusi24.rfid.base.BaseActivity;
import com.fusi24.rfid.config.Constant;
import com.fusi24.rfid.data.api.RestService;
import com.fusi24.rfid.data.api.RestServiceFactory;
import com.fusi24.rfid.data.entity.DataDetail;
import com.fusi24.rfid.data.entity.DataEntryPermit;
import com.fusi24.rfid.databinding.ActivityHomeBinding;
import com.fusi24.rfid.ui.ScanActivity;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import timber.log.Timber;

public class HomeActivity extends BaseActivity implements View.OnClickListener {

    private DataEntryPermit dataEntryPermit;
    private List<DataEntryPermit> entryPermitList;
    private List<DataDetail> statusScanList;
    private ActivityHomeBinding binding;
    Toolbar toolbar;
    TextView toolbarTitle;
    RadioButton rb;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityHomeBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);

        toolbar = findViewById(R.id.toolbar);
        toolbarTitle = findViewById(R.id.toolbar_text);

        if(toolbarTitle!=null && toolbar!=null) {
            toolbarTitle.setText(getString(R.string.app_name));
            setSupportActionBar(toolbar);
        }

        if (getSupportActionBar() != null){
            getSupportActionBar().setDisplayShowTitleEnabled(false);
        }

        entryPermitList = new ArrayList<>();
        statusScanList = new ArrayList<>();

        initView();
        getDataPermit();
        getDataStatusScan();
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.findItem(R.id.app_version)
                .setEnabled(false)
                .setTitle(BuildConfig.VERSION_NAME);
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_home, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {

        if (item.getItemId() == R.id.action_logout) {
            getSessionManager().clearData();
            startActivity(new Intent(this, SplashScreenActivity.class));
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onResume() {
        super.onResume();
        initEvent();
    }

    @SuppressLint("SetTextI18n")
    private void initView(){
        binding.tvRfidGunId.setText("RFID-" + Build.ID + "-" + Build.DEVICE);
        binding.tvRfidCardNumber.setText(getSessionManager().getUserRfidNumber());
        binding.tvEmployeeName.setText(getSessionManager().getEmployeeName());
        //TODO STILL HARDCODE
        binding.tvGateName.setText("Suzuran");
        binding.tvSiteName.setText("BMO 1");
    }

    private void initEvent(){

        binding.spinnerPermit.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                dataEntryPermit = (DataEntryPermit) parent.getItemAtPosition(position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                dataEntryPermit = (DataEntryPermit) parent.getSelectedItem();
            }
        });

        binding.btnScan.setOnClickListener(v -> {

            if (binding.spinnerPermit.getSelectedItem() == null){
                Toast.makeText(this, "Data Permit Belum Terpilih", Toast.LENGTH_SHORT).show();
                return;
            }

            Intent intent = new Intent(this, ScanActivity.class);
            intent.putExtra(ScanActivity.ID_PERMIT, String.valueOf(dataEntryPermit.getId()));
            startActivity(intent);
        });

        binding.rgStatusScan.setOnCheckedChangeListener((group, checkedId) -> Timber.i("DEVELOPER : %s", checkedId));

    }

    private void getDataPermit() {
        RestService restService = RestServiceFactory.create(Constant.JWT_TOKEN);
        Call<List<DataEntryPermit>> call = restService.getEntryPermit();
        call.enqueue(new Callback<List<DataEntryPermit>>() {
            @Override
            public void onResponse(@NotNull Call<List<DataEntryPermit>> call, @NotNull Response<List<DataEntryPermit>> response) {
                if (response.body() != null) {
                    entryPermitList.addAll(response.body());
                    setDataPermit();
                }
            }

            @Override
            public void onFailure(@NotNull Call<List<DataEntryPermit>> call, @NotNull Throwable t) {
                Toast.makeText(HomeActivity.this, "Failed to get data permit", Toast.LENGTH_SHORT).show();
                Timber.i("DEVELOPER : %s", t.getMessage());
            }
        });
    }

    private void getDataStatusScan(){
        RestService restService = RestServiceFactory.create(Constant.JWT_TOKEN);
        Call<List<DataDetail>> call = restService.getLookUp("BeSCAN_CHECK_TYPE");
        call.enqueue(new Callback<List<DataDetail>>() {
            @Override
            public void onResponse(@NotNull Call<List<DataDetail>> call, @NotNull Response<List<DataDetail>> response) {
                if (response.body() != null){
                    statusScanList.addAll(response.body());
                    setDataStatusScan();
                }
            }

            @Override
            public void onFailure(@NotNull Call<List<DataDetail>> call, @NotNull Throwable t) {
                Toast.makeText(HomeActivity.this, "Failed to get data status Scan", Toast.LENGTH_SHORT).show();
                Timber.i("DEVELOPER : %s", t.getMessage());
            }
        });
    }

    private void setDataPermit(){
        ArrayAdapter<DataEntryPermit> adapter = new ArrayAdapter<>(this, android.R.layout.simple_spinner_dropdown_item, entryPermitList);
        binding.spinnerPermit.setAdapter(adapter);
    }

    private void setDataStatusScan(){
        binding.rgStatusScan.setOrientation(LinearLayout.VERTICAL);
        for (int i=0; i<statusScanList.size(); i++){
            rb = new RadioButton(this);
            rb.setId(View.generateViewId());
            rb.setText(statusScanList.get(i).getName());
            rb.setOnClickListener(this);
            binding.rgStatusScan.addView(rb);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        binding = null;
    }

    @Override
    public void onClick(View v) {

        rb.setOnClickListener(v1 -> {
            Timber.i("DEVELOPER RB : %s", v1.getId());
        });

        Timber.tag(HomeActivity.class.getName()).d("DEVELOPER : Name " + ((RadioButton) v).getText() + " Id is " + v.getId());
    }
}
